
#include "app_csitx_module.h"

vx_status app_init_csitx(vx_context context, CsitxObj *csitxObj, char *objName)
{
	vx_status status = VX_SUCCESS;
	uint32_t loopCnt;
  vx_uint32 i = 0;
	vx_image input_image;

	if (vx_true_e == tivxIsTargetEnabled(TIVX_TARGET_CSITX))
	{
		status = VX_SUCCESS;
	}
	else
	{
		printf("app_init_capture failed!! line:%d\n", __LINE__);
		status = VX_FAILURE;
	}

  csitxObj->width 		= DISPLAY_WIDTH;
  csitxObj->height 		= DISPLAY_HEIGHT;
  /*csitxObj->format 		= VX_DF_IMAGE_RGBX;*/
  csitxObj->format 		= VX_DF_IMAGE_NV12;

  /* CSITX Config initialization */
  tivx_csitx_params_init(&csitxObj->cstix_params);
  csitxObj->cstix_params.numInst                          = 1U;
  csitxObj->cstix_params.numCh                            = 0; //1
  csitxObj->cstix_params.instId[0U]                       = CSITX_INST_ID;
  csitxObj->cstix_params.instCfg[0U].rxCompEnable         = (uint32_t)vx_true_e;
  csitxObj->cstix_params.instCfg[0U].rxv1p3MapEnable      = (uint32_t)vx_true_e;
  csitxObj->cstix_params.instCfg[0U].laneBandSpeed        = CSITX_LANE_BAND_SPEED;
  csitxObj->cstix_params.instCfg[0U].laneSpeedMbps        = CSITX_LANE_SPEED_MBPS;
  csitxObj->cstix_params.instCfg[0U].numDataLanes         = 5U;//2u
  csitxObj->cstix_params.instCfg[0U].fifoDepth         = 192;

  for (loopCnt = 0U ;
      loopCnt < csitxObj->cstix_params.instCfg[0U].numDataLanes ;
      loopCnt++)
  {
      csitxObj->cstix_params.instCfg[0U].lanePolarityCtrl[loopCnt] = 0u;
  }
  for (loopCnt = 0U; loopCnt < NUM_CHANNELS; loopCnt++)
  {
      csitxObj->cstix_params.chVcNum[loopCnt]   = loopCnt;
      csitxObj->cstix_params.chInstMap[loopCnt] = CSITX_INST_ID;
  }

  printf("csitx ins %d %d %d\n",csitxObj->cstix_params.instId[0U],csitxObj->cstix_params.instCfg[0U].laneSpeedMbps,csitxObj->cstix_params.instCfg[0U].numDataLanes);

  csitxObj->csitx_params_obj = vxCreateUserDataObject(context, "tivx_csitx_params_t", sizeof(tivx_csitx_params_t), &csitxObj->cstix_params);

	input_image = vxCreateImage(context, csitxObj->width, csitxObj->height, csitxObj->format);
	for (i = 0; i < APP_MAX_BUFQ_DEPTH ; i++)
	{
  	csitxObj->input_arr[i] = vxCreateObjectArray(context, (vx_reference)input_image, 1);
		csitxObj->input_image[i] = (vx_image)vxGetObjectArrayItem((vx_object_array)csitxObj->input_arr[i], 0);
	}
	vxReleaseImage(&input_image);

	status = vxGetStatus((vx_reference)csitxObj->csitx_params_obj);

  return status;
}

vx_status app_create_graph_csitx(vx_graph graph, CsitxObj *csitxObj, vx_object_array input_arr)
{
	vx_status status = VX_SUCCESS;

	csitxObj->node = tivxCsitxNode(graph, csitxObj->csitx_params_obj, input_arr);
	vxSetNodeTarget(csitxObj->node, VX_TARGET_STRING, TIVX_TARGET_CSITX);
	status = vxGetStatus((vx_reference)csitxObj->node);

	return status;
}

void app_delete_csitx(CsitxObj *csitxObj)
{
  if(csitxObj->node != NULL)
  {
    vxReleaseNode(&csitxObj->node);
  }
}

void app_deinit_csitx(CsitxObj *csitxObj)
{
  vx_uint32 i = 0;
	vxReleaseUserDataObject(&csitxObj->csitx_params_obj);

	for (i = 0; i < APP_MAX_BUFQ_DEPTH; i++)
	{
		vxReleaseImage(&csitxObj->input_image[i]);
		vxReleaseObjectArray(&csitxObj->input_arr[i]);
	}
}
